from __future__ import annotations

from enum import unique, Enum


@unique
class MessageDirection(Enum):
    INCOMING = 2
    OUTGOING = 1

    @classmethod
    def from_value(cls, v: str | int) -> MessageDirection:
        if v == "1" or v == 1:
            return MessageDirection.OUTGOING
        if v == "2" or v == 2:
            return MessageDirection.INCOMING
        else:
            raise ValueError(f"Value '{v}' is unknown for MessageDirection.")



from pythontextnow.api.TextNowAPI import TextNowAPI
from pythontextnow.enum import MessageDirection
from pythontextnow.model.Message import Message
from pythontextnow.util import general


def send_sms(*, message: str, send_to: str):
    """
    Sends an sms text message to this number.
    """
    message = general.replace_newlines(message)
    text_now_api = TextNowAPI()
    text_now_api.send_message(message=message, send_to=send_to)


def get_all_messages() -> list[Message]:
    """
    This gets the last 30 sent and received messages.
    """
    text_now_api = TextNowAPI()
    return text_now_api.get_all_messages()


def get_all_incoming_messages() -> list[Message]:
    """
    Gets all incoming messages.
    """
    all_messages = get_all_messages()
    return [message for message in all_messages if message.direction == MessageDirection.INCOMING]


def get_all_outgoing_messages() -> list[Message]:
    """
    This all messages sent by your account.
    """
    all_messages = get_all_messages()
    return [message for message in all_messages if message.direction == MessageDirection.OUTGOING]


def get_all_unread_messages():
    """
    Gets all unread messages.
    """
    all_messages = get_all_messages()
    return [message for message in all_messages if not message.read]


def mark_as_read(*, message: Message = None, messages: list[Message] = None) -> None:
    """
    Marks the given message/s as read.
    """
    if message is None and messages is None:
        raise ValueError("'message' and 'messages' cannot both be None.")
    all_messages = messages
    if all_messages is None:
        all_messages = [message]
    text_now_api = TextNowAPI()
    for message in all_messages:
        text_now_api.mark_message_as_read(message)



from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime

from pythontextnow.enum import MessageType, MessageDirection


@dataclass(kw_only=True)
class Message:
    number: str
    date: datetime
    first_contact: bool
    type_: MessageType
    read: bool
    id_: str
    direction: MessageDirection
    raw: dict

    @classmethod
    def from_dict(cls, message_dict: dict) -> Message:
        number = message_dict["contact_value"]
        date = datetime.fromisoformat(message_dict["date"].replace("Z", "+00:00"))
        first_contact = message_dict["conversation_filtering"]["first_time_contact"]
        type_ = MessageType.from_value(message_dict["message_type"])
        read = message_dict["read"]
        id_ = message_dict["id"]
        direction = MessageDirection.from_value(message_dict["message_direction"])
        raw = message_dict
        return Message(number=number,
                       date=date,
                       first_contact=first_contact,
                       type_=type_,
                       read=read,
                       id_=id_,
                       direction=direction,
                       raw=raw)



from __future__ import annotations

from dataclasses import dataclass

from pythontextnow.model.Message import Message


@dataclass
class MultiMediaMessage(Message):
    media: str

    @classmethod
    def from_dict(cls, message_dict: dict) -> MultiMediaMessage:
        message = Message.from_dict(message_dict)
        return MultiMediaMessage(number=message.number,
                                 date=message.date,
                                 first_contact=message.first_contact,
                                 type_=message.type_,
                                 read=message.read,
                                 id_=message.id_,
                                 direction=message.direction,
                                 raw=message.raw,
                                 media=message_dict["message"])



from __future__ import annotations

from enum import Enum, unique


@unique
class MessageType(Enum):
    MULTIMEDIA = 2
    TEXT = 1

    @classmethod
    def from_value(cls, v: str | int) -> MessageType:
        if v == "1" or v == 1:
            return MessageType.TEXT
        if v == "2" or v == 2:
            return MessageType.MULTIMEDIA
        else:
            raise ValueError(f"Value '{v}' is unknown for MessageType.")



from __future__ import annotations

from dataclasses import dataclass


@dataclass(kw_only=True, frozen=True)
class ClientConfig:
    """
    Used to hold the Client config.
    """
    username: str
    events: list
    headers: dict
    cookies: dict


class Client:
    """
    This class is used to store and set up initial Client configuration.
    """
    client_config = None

    @classmethod
    def set_client_config(cls, username: str, sid_cookie: str, csrf_cookie: str) -> None:
        from pythontextnow.api.TextNowAPI import TextNowAPI
        text_now_api = TextNowAPI()
        cookies = {
            "connect.sid": sid_cookie,
            "_csrf": csrf_cookie,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.104 Safari/537.36 ",
            "x-csrf-token": text_now_api.get_csrf_token(cookies)
        }

        client_config = ClientConfig(username=username,
                                     events=list(),
                                     headers=headers,
                                     cookies=cookies)
        cls.client_config = client_config

    @classmethod
    def get_client_config(cls) -> ClientConfig:
        return cls.client_config



from __future__ import annotations

from dataclasses import dataclass

from pythontextnow.model.Message import Message


@dataclass
class TextMessage(Message):
    text: str

    @classmethod
    def from_dict(cls, message_dict: dict) -> TextMessage:
        message = Message.from_dict(message_dict)
        return TextMessage(number=message.number,
                           date=message.date,
                           first_contact=message.first_contact,
                           type_=message.type_,
                           read=message.read,
                           id_=message.id_,
                           direction=message.direction,
                           raw=message.raw,
                           text=message_dict["message"])



import json
from datetime import datetime
from urllib.parse import quote

import requests

from pythontextnow.api.Client import Client, ClientConfig
from pythontextnow.enum import MessageType, MessageDirection, ContactType, ReadStatus
from pythontextnow.model.Message import Message
from pythontextnow.model.MultiMediaMessage import MultiMediaMessage
from pythontextnow.model.TextMessage import TextMessage
from pythontextnow.util.ConfigReader import ConfigReader


class TextNowAPI:
    def __init__(self):
        self.__BASE_URL = ConfigReader.get("api", "textnow_base_url")
        self.__API_ROUTE = ConfigReader.get("api", "api_route")
        self.__CONVERSATIONS_ROUTE = ConfigReader.get("api", "conversations_route")
        self.__MESSAGES_ROUTE = ConfigReader.get("api", "messages_route")
        self.__MESSAGING_ROUTE = ConfigReader.get("api", "messaging_route")
        self.__USERS_ROUTE = ConfigReader.get("api", "users_route")

    @property
    def __client_config(self) -> ClientConfig:
        return Client.get_client_config()

    def get_csrf_token(self, cookies: dict) -> str:
        response = requests.get(f"{self.__BASE_URL}{self.__MESSAGING_ROUTE}",
                                cookies=cookies)

        response.raise_for_status()

        resp = response.text
        needle = 'csrf-token" content="'
        needle_index = resp.find(needle)
        token_start = needle_index + len(needle)
        token_end = resp.find('"', token_start)
        csrf_token = resp[token_start:token_end]
        return csrf_token

    def send_message(self, *, message: str, send_to: str) -> None:
        json_data = {"contact_value": send_to,
                     "contact_type": ContactType.DEFAULT.value,
                     "message": message,
                     "read": ReadStatus.READ.value,
                     "message_direction": MessageDirection.OUTGOING.value,
                     "message_type": MessageType.MULTIMEDIA.value,
                     "from_name": self.__client_config.username,
                     "has_video": False,
                     "new": True,
                     "date": datetime.now().isoformat()}

        data = {"json": json.dumps(json_data)}

        response = requests.post(
            f"{self.__BASE_URL}{self.__API_ROUTE}{self.__USERS_ROUTE}/{self.__client_config.username}{self.__MESSAGES_ROUTE}",
            headers=self.__client_config.headers,
            cookies=self.__client_config.cookies,
            data=data)
        response.raise_for_status()

    def get_all_messages(self) -> list[Message]:
        response = requests.get(
            f"{self.__BASE_URL}{self.__API_ROUTE}{self.__USERS_ROUTE}/{self.__client_config.username}{self.__MESSAGES_ROUTE}",
            headers=self.__client_config.headers,
            cookies=self.__client_config.cookies)
        response.raise_for_status()

        message_dicts = json.loads(response.content)["messages"]

        all_messages = list()
        # sort into Text and MultiMedia messages
        for message_dict in message_dicts:
            message_type = MessageType.from_value(message_dict["message_type"])
            if message_type == MessageType.TEXT:
                all_messages.append(TextMessage.from_dict(message_dict))
            elif message_type == MessageType.MULTIMEDIA:
                all_messages.append(MultiMediaMessage.from_dict(message_dict))
        return all_messages

    def mark_message_as_read(self, message: Message) -> None:

        clean_number = quote(message.number)
        url = f"{self.__BASE_URL}{self.__API_ROUTE}{self.__USERS_ROUTE}/{self.__client_config.username}{self.__CONVERSATIONS_ROUTE}/{clean_number}"

        params = {
            "latest_message_id": message.id_,
            "http_method": "PATCH"
        }

        data = {"read": True}

        response = requests.post(url,
                                 params=params,
                                 data=data,
                                 cookies=self.__client_config.cookies,
                                 headers=self.__client_config.headers)
        response.raise_for_status()
